// Copyright (C) 2021 THL A29 Limited, a Tencent company.  All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software distributed under the License
// is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions and limitations under
// the License.
//
// ╔════════════════════════════════════════════════════════════════════════════════════════╗
// ║──█████████╗───███████╗───████████╗───██╗──────██╗───███████╗───████████╗───████████╗───║
// ║──██╔══════╝──██╔════██╗──██╔════██╗──██║──────██║──██╔════██╗──██╔════██╗──██╔════██╗──║
// ║──████████╗───██║────██║──████████╔╝──██║──█╗──██║──█████████║──████████╔╝──██║────██║──║
// ║──██╔═════╝───██║────██║──██╔════██╗──██║█████╗██║──██╔════██║──██╔════██╗──██║────██║──║
// ║──██║─────────╚███████╔╝──██║────██║──╚████╔████╔╝──██║────██║──██║────██║──████████╔╝──║
// ║──╚═╝──────────╚══════╝───╚═╝────╚═╝───╚═══╝╚═══╝───╚═╝────╚═╝──╚═╝────╚═╝──╚═══════╝───║
// ╚════════════════════════════════════════════════════════════════════════════════════════╝
//
// Authors: Aster JIAN (asterjian@qq.com)
//          Yzx (yzxyzxyzx777@outlook.com)
//          Ao LI (346950981@qq.com)
//          Paul LU (lujq96@gmail.com)

#pragma once

#include <memory>
#include <string>
#include <vector>

#include "fwd_torch/torch_cvt/torch_desc_creators/i_torch_layer_creator.h"
#include "fwd_torch/torch_cvt/torch_helper.h"

FWD_TORCH_NAMESPACE_BEGIN

/**
 * \brief Embedding 层描述创建器
 */
template <>
class TLayerDescCreator<TrtGatherDesc> : public ILayerDescCreator {
 public:
  bool Check(const JitNode* node, const TorchModule& module) override {
    return node->kind() == c10::aten::embedding;
  }

  std::shared_ptr<TrtLayerDesc> Create(const JitNode* node, const TorchModule& module,
                                       std::vector<const JitValue*>& input_values) override {
    LOG(INFO) << "TrtGatherDesc::Create";
    auto layer_desc = std::make_shared<TrtGatherDesc>();

    const auto inputs = node->inputs();

    // Input 0, weights
    // Input 1, indices
    // Input 2, kind = int, padding_idx
    // Input 3, kind = bool, scale_grad_by_freq
    // Input 4, kind = bool, sparse
    input_values.push_back(inputs[1]);

    T_CHECK_EQ(module.Get(inputs[3]).toBool(), false);
    T_CHECK_EQ(module.Get(inputs[4]).toBool(), false);

    const auto weights = module.Get(inputs[0]).toTensor();
    layer_desc->inputs[0].data = ToFwdWeights(weights);
    layer_desc->inputs[0].inUse = true;
    layer_desc->inputs[0].dim = DimsOf(weights);
    layer_desc->inputs[1].inUse = false;
    layer_desc->gatherAxis = 0;  // TODO(Ao Li): 这里固定
    return layer_desc;
  }
};

FWD_TORCH_NAMESPACE_END
